/*
 *      This program is free software; you can redistribute it and/or modify
 *      it under the terms of the GNU General Public License as published by
 *      the Free Software Foundation; either version 2 of the License, or
 *      (at your option) any later version.
 *      
 *      This program is distributed in the hope that it will be useful,
 *      but WITHOUT ANY WARRANTY; without even the implied warranty of
 *      MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *      GNU General Public License for more details.
 *      
 *      You should have received a copy of the GNU General Public License
 *      along with this program; if not, write to the Free Software
 *      Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston,
 *      MA 02110-1301, USA.
 */

//control_window.h
//Written by Cosmin Deaconu <cozzyd@ccrma.stanford.edu> 

/* A ControlWindow is the inspector window that allows a user to select the
 * currently used plugin and to modify settable plugin options */

#ifndef CONTROL_WINDOW_H
#define CONTROL_WINDOW_H

#include <gtkmm/window.h>
#include <gtkmm/comboboxtext.h>
#include <gtkmm/box.h>
#include <gtkmm/image.h>
#include <gtkmm/scale.h>
#include <gtkmm/aboutdialog.h>
#include <gtkmm/inputdialog.h>
#include "plugin_loader.h"
#include "input_window.h"
#include <vector>

#define VERSION "0.2"


class ControlWindow : public Gtk::Window {
  
  public:
    /* Creates a new control window associated with the given InputWindow */ 
    ControlWindow(InputWindow *); 
    ~ControlWindow(); 
    
  
  protected:
    bool on_delete_event(GdkEventAny * event);

    
    Gtk::VBox mainBox;            // Holds all the widgets
    Gtk::VBox controlBox;         // Holds all dynamically loaded per 
                                  //plugin widgets
    Gtk::Frame controlFrame;      // Frame around controlBox
    
    Gtk::HBox buttonBox;          // About and Extended Input dialogs
    Gtk::Button showInput;        //Button for extended input
    Gtk::Button showAbout;        //Button for about 
    
    Gtk::HBox titleBox;           //Holds the name and icon
    Gtk::Image logo;              //The icon
    Gtk::Label title;             //The titleglib loading resources relative paths
    
    Gtk::HBox volumeBox;          //Holds the volume control
    Gtk::Label volumeLabel;
    Gtk::HScale volumeControl; 
    
    Gtk::HBox pluginSelectorBox;  //Holds the plugin selector
    Gtk::Label pluginLabel; 
    Gtk::ComboBoxText chooser;
    
  
    Gtk::InputDialog inputDialog; //Extended input dialog
    
    InputWindow * inputWindow;    //The InputWindow... see input_window.h
  
    std::vector<Gtk::Widget *> addedWidgets; //Keeps track of added widgets
                                             //from plugins so we can remove 
                                             // them. 
                                             
    std::vector <PluginFileName*> plugins;  //A list of plugin filenames
    destroy_t * destroyFn; //Destroy function for the current plugin
    void * oldLoad; // void * returned by dlopen for the current plugin
    
    //Methods... described in .cpp
    void clickInputButton(); 
    void clickAboutButton(); 
    void hideInput(); 
    void pluginSelected();
    void volumeChanged(); 
    void loadPlugins(); 
    void loadPlugin(int); 
    void updateControl();
};



#endif
