/*
 *      This program is free software; you can redistribute it and/or modify
 *      it under the terms of the GNU General Public License as published by
 *      the Free Software Foundation; either version 2 of the License, or
 *      (at your option) any later version.
 *      
 *      This program is distributed in the hope that it will be useful,
 *      but WITHOUT ANY WARRANTY; without even the implied warranty of
 *      MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *      GNU General Public License for more details.
 *      
 *      You should have received a copy of the GNU General Public License
 *      along with this program; if not, write to the Free Software
 *      Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston,
 *      MA 02110-1301, USA.
 */

//input_window.h
//Written by Cosmin Deaconu <cozzyd@ccrma.stanford.edu> 

#ifndef INPUT_WINDOW_H
#define INPUT_WINDOW_H

#include <gtkmm/window.h>
#include <gtkmm/drawingarea.h>
#include <gdkmm/types.h>
#include <gdk/gdk.h>
#include <gtkmm/aspectframe.h>
#include <gtkmm/statusbar.h>
#include <gtkmm/box.h>
#include "tabsynth_plugin.h"
#include "resources/icon.h"

#define DEFAULT_SIZE 400

/* The InputWindow class represents the window that is responsible for
 * motion detection. On update events from the windowing system, it passes
 * the relevant data to the currently enabled plugin. 
 */

class InputWindow : public Gtk::Window
{
  public:
    InputWindow(); 
    ~InputWindow(); 
    void setPlugin(TSPlugin * ); 
    TSPlugin * getPlugin(); 
    
    /*FIXME: Not used yet */ 
    void enableStatusBar(bool); 
    
  protected:
  
    bool motionHappened(GdkEventMotion *); 
    bool buttonPressed(GdkEventButton *); 
    bool exposeHappened(GdkEventExpose *); 
    void eventHappened(GdkEventButton *); 
    bool statusEnabled; 
    
  	TSPlugin * plugin; 
    Gtk::DrawingArea drawingArea; 
    Gtk::AspectFrame frame; 
    Gtk::VBox box; 
    
    Gtk::HBox statusbox; 
    Gtk::Statusbar bar1; 
    Gtk::Statusbar bar2; 
    Gtk::Statusbar bar3; 
    Gtk::Statusbar bar4; 


    
};
#endif
