/*
 *      This program is free software; you can redistribute it and/or modify
 *      it under the terms of the GNU General Public License as published by
 *      the Free Software Foundation; either version 2 of the License, or
 *      (at your option) any later version.
 *      
 *      This program is distributed in the hope that it will be useful,
 *      but WITHOUT ANY WARRANTY; without even the implied warranty of
 *      MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *      GNU General Public License for more details.
 *      
 *      You should have received a copy of the GNU General Public License
 *      along with this program; if not, write to the Free Software
 *      Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston,
 *      MA 02110-1301, USA.
 */

// plugin_loader.cpp
// Written by Cosmin Deaconu <cozzyd@ccrma.stanford.edu>

#include "plugin_loader.h"
#include <glibmm/fileutils.h>
#include <glibmm.h>
#include <iostream>
#include <malloc.h>


static bool alreadyInList(std::vector<PluginFileName*> * plugins, 
													PluginFileName * test) {
														
	for (int i = 0; i < plugins->size(); i++) {
		if ((*plugins)[i]->name==test->name)
			return true;
	}														

	return false; 
}
												

std::vector<PluginFileName*> * load_plugins() {
	
	std::vector<PluginFileName*> * plugins = 
		new std::vector<PluginFileName*>(); 
	
	for (int i = 0; i < NUM_PLUGIN_DIRS; i++) {
		try {
			
			std::string dir_name = PLUGIN_DIRS[i];
			
			//Replace ~ with correct home directory
			int home_pos = dir_name.find("~");
			if (home_pos != std::string::npos) {
				dir_name.replace(home_pos,1,Glib::get_home_dir()); 
			}
			
			Glib::Dir dir(dir_name);

			for (std::string name = dir.read_name(); 
				 name!=""; 
				 name = dir.read_name()) {
			
				//Ensure .so //TODO: Make more robust
				if (name.find(".so")==std::string::npos)
					continue; 
				
				
				PluginFileName* this_plugin = new PluginFileName();
				this_plugin->name =  name ; 
				this_plugin->dir = dir_name ; 
				
				std::cout << "Found plugin "<<dir_name<<name <<std::endl;

				//ensure not already in the plugin list
				if (alreadyInList(plugins,this_plugin))
					continue; 
				
				plugins->push_back(this_plugin); 
				
			}
			
			
		} catch (Glib::FileError err) {
			std::cout << err.what() <<std::endl;
			continue; 
		}
		
	}
	
	return plugins; 
	
}

