/*
 *      This program is free software; you can redistribute it and/or modify
 *      it under the terms of the GNU General Public License as published by
 *      the Free Software Foundation; either version 2 of the License, or
 *      (at your option) any later version.
 *      
 *      This program is distributed in the hope that it will be useful,
 *      but WITHOUT ANY WARRANTY; without even the implied warranty of
 *      MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *      GNU General Public License for more details.
 *      
 *      You should have received a copy of the GNU General Public License
 *      along with this program; if not, write to the Free Software
 *      Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston,
 *      MA 02110-1301, USA.
 */

//plugin_option.h
//Written by Cosmin Deaconu <cozzyd@ccrma.stanford.edu> 


/* A PluginOption represents a parameter settable via the GUI. Each plugin
 * contains a list of PluginOptions which are loaded into the ControlWindow
 * automagically*/

#ifndef PLUGIN_OPTION_H
#define PLUGIN_OPTION_H

#include <string>


/* Represents the type of option */
enum pluginOptionType {
  OPTION_SLIDER, /*A slider */
  OPTION_SPIN, /* A spin button */
  OPTION_CHECK, /*A check box. Note that these should only have values 0 and 1! */ 
};

class PluginOption
{
  
  public:
    PluginOption(pluginOptionType type,       //The type of option
                 std::string name,            //The label which will be displayed
                 std::string description,     //Tooltip text of label
                 double def = 0,              //Default value
                 double min = 0,              //Minimum value 
                 double max = 1,              //Maximum value
                 int numSteps = 10);          //Number of steps
                 
    pluginOptionType getType();     
    double getValue(); 
    std::string getName(); 
    std::string getDescription(); 
    void setValue(double); 
    double getMin(); 
    double getMax(); 
    int getNumSteps(); 
    ~PluginOption();
    
  protected:
    pluginOptionType type; 
    double min; 
    double max; 
    double value; 
    std::string name;
    std::string description; 
    int numSteps; 

};

#endif
