//drumz.cpp
//Written by Cosmin Deaconu <cozzyd@ccrma.stanford.edu> 


#include <math.h>
#include <stk/Drummer.h>
#include "../tabsynth_plugin.h"
#include <vector>

/* An attempt at a drumset. */

#define FONT_SIZE 12
#define PI 3.14159265358979   //Pi...


//snare drum
#define SNARE_X 0.3
#define SNARE_Y 0.68
#define SNARE_R 0.15
#define SNARE_F 74
#define SNARE_F2 0
#define SNARE_N "Snare"

//hihat
#define HH_X 0.18
#define HH_Y 0.4
#define HH_R 0.15
#define HH_F 104
#define HH_F2 0
#define HH_N "Hihat"

//bass
#define BASS_X 0.5
#define BASS_Y 0.5
#define BASS_R 0.1
#define BASS_F 66
#define BASS_F2 0
#define BASS_N "Bass"

//hitom
#define HT_X 0.399
#define HT_Y 0.272
#define HT_R 0.1
#define HT_F 131
#define HT_F2 0
#define HT_N "Hi Tom"

//midtom
#define MT_X 0.601
#define MT_Y 0.272
#define MT_R 0.1
#define MT_F 124
#define MT_F2 0
#define MT_N "Mid Tom"

//floortom
#define FT_X 0.7
#define FT_Y 0.68
#define FT_R 0.15
#define FT_F 88
#define FT_F2 0
#define FT_N "Floor Tom"

//crash/ride cymbal
#define CYMBAL_X 0.82
#define CYMBAL_Y 0.4
#define CYMBAL_R 0.15
#define CYMBAL_F 139
#define CYMBAL_F2 117
#define CYMBAL_N "Cymbal"



class Drum {
  
  public:
    Drum(float center_x, float center_y, float radius, StkFloat freq, StkFloat alt_freq, std::string display_name) {
      x = center_x;
      y = center_y;
      r = radius; 
      r2 = pow(radius,2);
      f = freq; 
      f2 = alt_freq; 
      name = display_name;
    }
    
  std::string name; 
  float x;
  float y; 
  float r2; 
  float r; 
  StkFloat f; 
  StkFloat f2;
  
  bool contains(float test_x, float test_y) {
    float dist = pow(test_x - x,2) + pow(test_y-y,2);
    return (dist <= r2); 
  }
};


class Drumz : public TSPlugin {  
  
  protected:
    Drummer * drumz; //The main instrument
    std::vector<Drum> drums; 
    double vol; 
    bool on; 
    
  public: 
    //Constructor
    Drumz() {
      drumz = new Drummer(); 
      on = false;
      vol = 0; 
      
      //set up different drums
      drums.push_back(Drum(SNARE_X, SNARE_Y, SNARE_R, SNARE_F, SNARE_F2, SNARE_N));     
      drums.push_back(Drum(HH_X, HH_Y, HH_R, HH_F, HH_F2, HH_N));   
      drums.push_back(Drum(BASS_X, BASS_Y, BASS_R, BASS_F, BASS_F2, BASS_N));   
      drums.push_back(Drum(HT_X, HT_Y, HT_R, HT_F, HT_F2, HT_N));   
      drums.push_back(Drum(MT_X, MT_Y, MT_R, MT_F, MT_F2, MT_N));   
      drums.push_back(Drum(FT_X, FT_Y, FT_R, FT_F, FT_F2, FT_N));   
      drums.push_back(Drum(CYMBAL_X, CYMBAL_Y, CYMBAL_R, CYMBAL_F, CYMBAL_F2, CYMBAL_N));   
      
    }
    
    ~Drumz() {
      delete drumz; 
    }
    
    StkFloat setParameters(double x, double y, double pressure, 
                            bool button2, bool button3, double tiltX, 
                            double tiltY) {
      
      StkFloat freq = 0;  
      
      if(!on && pressure > 0) {
        
        bool drumHit = false;
        
        //See if any drums are hit
        for (int i = 0; i < drums.size(); i++) {
          Drum drum = drums[i]; 
          if (drum.contains(x,y)) {
            drumHit = true; 
            freq = drum.f;
            if (button3 && drum.f2)
              freq=drum.f2;
            break;  
          }  
        }
        if (!drumHit) {
          on = false; 
          return 0; 
        }
        on = true; 
        drumz->noteOn(freq,pressure); 
      }
      
      if (on && pressure == 0) {
        on = false; 
      }
          
      return freq; 
    }

    StkFloat tick()  {
      return drumz->tick();
    }
    
    /* Draw some circles to make it easier to see what's going on */
    void draw(Cairo::RefPtr<Cairo::Context> cr, int size) {

      cr->set_line_width(size * 0.005); 
      //First draw the circles
      for (int i =0; i < drums.size(); i++) {
        Drum drum = drums[i];
        
        cr->arc(size*drum.x, size*drum.y, size*drum.r, 0, 2*PI); 
        cr->set_source_rgb( 0.94902 ,0.73333,0.31765);
        cr->fill_preserve(); 
        cr->set_source_rgb(0,0,0);
        cr->stroke(); 
      
      }
      
      //Then draw the text
      for (int i = 0; i < drums.size(); i++) {
        Drum drum = drums[i];
        
        cr->move_to((size*drum.x) - drum.name.length()*FONT_SIZE/3., size*drum.y + FONT_SIZE/2.);
        cr->set_font_size(12); 
        cr->show_text(drum.name);       
      }   
      
    }
   
    std::string  getName() {
      return "Drumz"; 
    }
    std::string  getDescription() {
      return "A graphical drum set. \n\nSometimes the drum waves don't load the first time (I think this is an STK bug). If a drum doesn't work, try a few other ones and come back to it.\n\nFor the cymbal, hold button3 (RMB) for the ride sound.\n\nIf this crashes when you hit a drum, then TabSynth can't find your rawwave path. TabSynth searches the following: /usr/local/share/stk/rawwaves, /usr/share/stk/rawwaves/, ./rawwaves/, ~/stk/rawwaves"; 
    }

    std::string getVersion() {
      return "0.1"; 
    }
};

/* In addition to containing a class inheriting from TSPlugin, 
 * a plugin must have the following two methods. */

extern "C" TSPlugin * init() {
  return new Drumz; 
}

extern "C" void destroy(TSPlugin * p) {
  delete p; 
}
