#include "../tabsynth_plugin.h" //This file must be included 
#include <stk/BlitSaw.h>       //Use this for synthesis
#include <cmath>

//super_simple.cpp

//Written by Cosmin Deaconu <cozzyd@ccrma.stanford.edu> 

/* A really dumb plugin for demonstration purposes. It is thoroughly 
 * explained in a tutorial in the documentation. */

class SuperSimple : public TSPlugin {
  
  protected: //Protected Member Variables
    double vol; 
    BlitSaw * blit;  
    
  public:
    SuperSimple() { //Constructor
      blit = new BlitSaw(220); 
      vol = 0; 
      
      //Add an option to set the number of harmonics 
      options.push_back(new PluginOption(OPTION_SPIN, "NumHarmonics", 
            "setHarmonics is called with this number", 10,1,100,50)); 
    }
  
    ~SuperSimple() { //Destructor
      delete blit;
    }
    
    //Tablet event occurred. Calculate new frequency from this. 
    //Return frequency for display purposes
    StkFloat setParameters(double x, double y, double pressure, 
                            bool button2, bool button3, double tiltX, 
                            double tiltY) {
  
      //Some weird way of computing frequency
      double freq = 1000 * pow((x*x - y*y + x*y - pressure*pressure),2) ;
      
      //Use pressure as volume      
      vol = pressure;  
      blit->setFrequency(freq); 
      
      //Grab the harmonics from the GUI option
      blit->setHarmonics( (unsigned int) options[0]->getValue()); 
      return freq; 
    }

    //Computes next sample
    StkFloat tick()  {
      return blit->tick() * vol; 
    }
    
    /* We could draw something here, but Cairo's complicated, so we'll skip this*/
    void draw(Cairo::RefPtr<Cairo::Context> cr, int size) {}
    
    std::string getName() {
      return "Super Simple"; 
    }
    std::string getDescription() {
      return "A toy plugin for demonstration"; 
    }

    std::string getVersion() {
      return "-1"; 
    }
}; //End of Class Definition

/* In addition to containing a class inheriting from TSPlugin, 
 * a plugin must have the following two methods. (Used for
 * dynamic loading) */

extern "C" TSPlugin * init() {
  return new SuperSimple; 
}

extern "C" void destroy(TSPlugin * p) {
  delete p; 
}
