/*
 *      This program is free software; you can redistribute it and/or modify
 *      it under the terms of the GNU General Public License as published by
 *      the Free Software Foundation; either version 2 of the License, or
 *      (at your option) any later version.
 *      
 *      This program is distributed in the hope that it will be useful,
 *      but WITHOUT ANY WARRANTY; without even the implied warranty of
 *      MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *      GNU General Public License for more details.
 *      
 *      You should have received a copy of the GNU General Public License
 *      along with this program; if not, write to the Free Software
 *      Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston,
 *      MA 02110-1301, USA.
 */

//tabsynth_plugin.cpp
//Written by Cosmin Deaconu <cozzyd@ccrma.stanford.edu> 

/* Implements the TSPlugin class */ 

#include "tabsynth_plugin.h"

#include <glibmm/fileutils.h>
#include <glibmm.h>


/* Initializes a plugin. Initializing involves:
 *     - Opening a Realtime Wave Out (mono)
 *     - Setting sample rate 
 *     - We gotta find the rawwave path... boo
 */
TSPlugin::TSPlugin() :
options()
 {

 Stk::setSampleRate(SAMPLERATE); 
 
 //This is kinda annoying, but we need to figure out the stk rawwave path
 //This only really needs to be done once in tabsynth... but it shouldn't take
 //too long so whatever...
 
 for (int i = 0; i < NUM_RAWWAVE_PATHS; i++) {
     std::string dir_name = RAWWAVE_PATHS[i];
			
			//Replace ~ with correct home directory
			int home_pos = dir_name.find("~");
			if (home_pos != std::string::npos) {
				dir_name.replace(home_pos,1,Glib::get_home_dir()); 
			}
			try {
			  Glib::Dir dir(dir_name);
        Stk::setRawwavePath(dir_name);
        break; 
      } catch(Glib::FileError err) {
        continue; 
      }
 }
 
  
  try {
    out = new RtWvOut(1); 
  } catch (StkError &) {
    std::cout << "Could not open wave out" << std::endl; 
    exit(2); 
  }
  
  this->playing = false; 
  this->volume = 1.0; 
  smooth = new OnePole(SMOOTH_Z); 
}


/* Uninitializes a plugin */ 
TSPlugin::~TSPlugin() {
    stop(); 
    delete out; 
    delete smooth; 
    /* Delete PluginOptions */ 
    while (options.size() > 0) {
      PluginOption * opt = options.back(); 
      options.pop_back(); 
      delete opt; 
    }
  }


/* Method run by the audio processing thread */ 
void TSPlugin::run() {
    
    while(this->playing) {
      for (int i = 0; i < 100; i++) {
        try {
          out->tick( smooth->tick(volume) * this->tick() ); 
        } catch (StkError &) {
          std::cout << "Trouble playing :(" << std::endl;
          exit(2);
        }
      }
    }  
    
    return; 
}

/* Spawns the audio processing thread */ 
void TSPlugin::start() {
  
  if (this->playing) return; 
  
  this->playing = true; 
  thread = Glib::Thread::create(sigc::mem_fun(*this, &TSPlugin::run), true); 
}

/* Stops the audio processing thread */ 
void TSPlugin::stop() {
  playing = false; 
}


void TSPlugin::setVolume(double vol) {
  volume = vol; 
}

std::vector<PluginOption *> * TSPlugin::getOptions() {
  return &options; 
}


//Stub functions... 

void TSPlugin::draw(Cairo::RefPtr<Cairo::Context> cr, int size) {};
std::string TSPlugin::getName() {return "Stub";}
std::string TSPlugin::getDescription() {return "Stub (aka I hate C++)";}
std::string TSPlugin::getVersion() {return "-1";}
StkFloat TSPlugin::tick() { return 0. ;}
StkFloat TSPlugin::setParameters(double x, double y, double pressure, 
                          bool button2, bool button3, double tiltX, 
                          double tiltY){};
